@@ -12,7 +12,7 @@ use syn::{
     ExprMacro, ExprMatch, ExprMethodCall, ExprParen, ExprPath, ExprRange, ExprReference,
     ExprRepeat, ExprReturn, ExprStruct, ExprTry, ExprTryBlock, ExprTuple, ExprType, ExprUnary,
     ExprUnsafe, ExprWhile, ExprYield, FieldValue, GenericMethodArgument, Index, Label, Member,
-    MethodTurbofish, RangeLimits, ReturnType, Stmt, Token, UnOp,
+    MethodTurbofish, PathArguments, RangeLimits, ReturnType, Stmt, Token, UnOp,
 };
 
 impl Printer {
@@ -22,26 +22,26 @@ impl Printer {
             Expr::Assign(expr) => self.expr_assign(expr),
             Expr::AssignOp(expr) => self.expr_assign_op(expr),
             Expr::Async(expr) => self.expr_async(expr),
-            Expr::Await(expr) => self.expr_await(expr),
+            Expr::Await(expr) => self.expr_await(expr, false),
             Expr::Binary(expr) => self.expr_binary(expr),
             Expr::Block(expr) => self.expr_block(expr),
             Expr::Box(expr) => self.expr_box(expr),
             Expr::Break(expr) => self.expr_break(expr),
-            Expr::Call(expr) => self.expr_call(expr),
+            Expr::Call(expr) => self.expr_call(expr, false),
             Expr::Cast(expr) => self.expr_cast(expr),
             Expr::Closure(expr) => self.expr_closure(expr),
             Expr::Continue(expr) => self.expr_continue(expr),
-            Expr::Field(expr) => self.expr_field(expr),
+            Expr::Field(expr) => self.expr_field(expr, false),
             Expr::ForLoop(expr) => self.expr_for_loop(expr),
             Expr::Group(expr) => self.expr_group(expr),
             Expr::If(expr) => self.expr_if(expr),
-            Expr::Index(expr) => self.expr_index(expr),
+            Expr::Index(expr) => self.expr_index(expr, false),
             Expr::Let(expr) => self.expr_let(expr),
             Expr::Lit(expr) => self.expr_lit(expr),
             Expr::Loop(expr) => self.expr_loop(expr),
             Expr::Macro(expr) => self.expr_macro(expr),
             Expr::Match(expr) => self.expr_match(expr),
-            Expr::MethodCall(expr) => self.expr_method_call(expr),
+            Expr::MethodCall(expr) => self.expr_method_call(expr, false),
             Expr::Paren(expr) => self.expr_paren(expr),
             Expr::Path(expr) => self.expr_path(expr),
             Expr::Range(expr) => self.expr_range(expr),
@@ -49,7 +49,7 @@ impl Printer {
             Expr::Repeat(expr) => self.expr_repeat(expr),
             Expr::Return(expr) => self.expr_return(expr),
             Expr::Struct(expr) => self.expr_struct(expr),
-            Expr::Try(expr) => self.expr_try(expr),
+            Expr::Try(expr) => self.expr_try(expr, false),
             Expr::TryBlock(expr) => self.expr_try_block(expr),
             Expr::Tuple(expr) => self.expr_tuple(expr),
             Expr::Type(expr) => self.expr_type(expr),
@@ -65,14 +65,25 @@ impl Printer {
         }
     }
 
-    fn subexpr(&mut self, expr: &Expr) {
+    pub fn expr_beginning_of_line(&mut self, expr: &Expr, beginning_of_line: bool) {
         match expr {
-            Expr::Await(expr) => self.subexpr_await(expr),
+            Expr::Await(expr) => self.expr_await(expr, beginning_of_line),
+            Expr::Field(expr) => self.expr_field(expr, beginning_of_line),
+            Expr::Index(expr) => self.expr_index(expr, beginning_of_line),
+            Expr::MethodCall(expr) => self.expr_method_call(expr, beginning_of_line),
+            Expr::Try(expr) => self.expr_try(expr, beginning_of_line),
+            _ => self.expr(expr),
+        }
+    }
+
+    fn subexpr(&mut self, expr: &Expr, beginning_of_line: bool) {
+        match expr {
+            Expr::Await(expr) => self.subexpr_await(expr, beginning_of_line),
             Expr::Call(expr) => self.subexpr_call(expr),
-            Expr::Field(expr) => self.subexpr_field(expr),
-            Expr::Index(expr) => self.subexpr_index(expr),
-            Expr::MethodCall(expr) => self.subexpr_method_call(expr),
-            Expr::Try(expr) => self.subexpr_try(expr),
+            Expr::Field(expr) => self.subexpr_field(expr, beginning_of_line),
+            Expr::Index(expr) => self.subexpr_index(expr, beginning_of_line),
+            Expr::MethodCall(expr) => self.subexpr_method_call(expr, beginning_of_line, false),
+            Expr::Try(expr) => self.subexpr_try(expr, beginning_of_line),
             _ => {
                 self.cbox(-INDENT);
                 self.expr(expr);
@@ -149,16 +160,16 @@ impl Printer {
         self.end();
     }
 
-    fn expr_await(&mut self, expr: &ExprAwait) {
+    fn expr_await(&mut self, expr: &ExprAwait, beginning_of_line: bool) {
         self.outer_attrs(&expr.attrs);
         self.cbox(INDENT);
-        self.subexpr_await(expr);
+        self.subexpr_await(expr, beginning_of_line);
         self.end();
     }
 
-    fn subexpr_await(&mut self, expr: &ExprAwait) {
-        self.subexpr(&expr.base);
-        self.zerobreak();
+    fn subexpr_await(&mut self, expr: &ExprAwait, beginning_of_line: bool) {
+        self.subexpr(&expr.base, beginning_of_line);
+        self.zerobreak_unless_short_ident(beginning_of_line, &expr.base);
         self.word(".await");
     }
 
@@ -204,14 +215,14 @@ impl Printer {
         }
     }
 
-    fn expr_call(&mut self, expr: &ExprCall) {
+    fn expr_call(&mut self, expr: &ExprCall, beginning_of_line: bool) {
         self.outer_attrs(&expr.attrs);
-        self.expr(&expr.func);
+        self.expr_beginning_of_line(&expr.func, beginning_of_line);
         self.call_args(&expr.args);
     }
 
     fn subexpr_call(&mut self, expr: &ExprCall) {
-        self.subexpr(&expr.func);
+        self.subexpr(&expr.func, false);
         self.call_args(&expr.args);
     }
 
@@ -284,16 +295,16 @@ impl Printer {
         }
     }
 
-    fn expr_field(&mut self, expr: &ExprField) {
+    fn expr_field(&mut self, expr: &ExprField, beginning_of_line: bool) {
         self.outer_attrs(&expr.attrs);
         self.cbox(INDENT);
-        self.subexpr_field(expr);
+        self.subexpr_field(expr, beginning_of_line);
         self.end();
     }
 
-    fn subexpr_field(&mut self, expr: &ExprField) {
-        self.subexpr(&expr.base);
-        self.zerobreak();
+    fn subexpr_field(&mut self, expr: &ExprField, beginning_of_line: bool) {
+        self.subexpr(&expr.base, beginning_of_line);
+        self.zerobreak_unless_short_ident(beginning_of_line, &expr.base);
         self.word(".");
         self.member(&expr.member);
     }
@@ -382,16 +393,16 @@ impl Printer {
         self.end();
     }
 
-    fn expr_index(&mut self, expr: &ExprIndex) {
+    fn expr_index(&mut self, expr: &ExprIndex, beginning_of_line: bool) {
         self.outer_attrs(&expr.attrs);
-        self.expr(&expr.expr);
+        self.expr_beginning_of_line(&expr.expr, beginning_of_line);
         self.word("[");
         self.expr(&expr.index);
         self.word("]");
     }
 
-    fn subexpr_index(&mut self, expr: &ExprIndex) {
-        self.subexpr(&expr.expr);
+    fn subexpr_index(&mut self, expr: &ExprIndex, beginning_of_line: bool) {
+        self.subexpr(&expr.expr, beginning_of_line);
         self.word("[");
         self.expr(&expr.index);
         self.word("]");
@@ -464,22 +475,30 @@ impl Printer {
         self.end();
     }
 
-    fn expr_method_call(&mut self, expr: &ExprMethodCall) {
+    fn expr_method_call(&mut self, expr: &ExprMethodCall, beginning_of_line: bool) {
         self.outer_attrs(&expr.attrs);
         self.cbox(INDENT);
-        self.subexpr_method_call(expr);
+        let unindent_call_args = beginning_of_line && is_short_ident(&expr.receiver);
+        self.subexpr_method_call(expr, beginning_of_line, unindent_call_args);
         self.end();
     }
 
-    fn subexpr_method_call(&mut self, expr: &ExprMethodCall) {
-        self.subexpr(&expr.receiver);
-        self.zerobreak();
+    fn subexpr_method_call(
+        &mut self,
+        expr: &ExprMethodCall,
+        beginning_of_line: bool,
+        unindent_call_args: bool,
+    ) {
+        self.subexpr(&expr.receiver, beginning_of_line);
+        self.zerobreak_unless_short_ident(beginning_of_line, &expr.receiver);
         self.word(".");
         self.ident(&expr.method);
         if let Some(turbofish) = &expr.turbofish {
             self.method_turbofish(turbofish);
         }
+        self.cbox(if unindent_call_args { -INDENT } else { 0 });
         self.call_args(&expr.args);
+        self.end();
     }
 
     fn expr_paren(&mut self, expr: &ExprParen) {
@@ -560,14 +579,14 @@ impl Printer {
         self.word("}");
     }
 
-    fn expr_try(&mut self, expr: &ExprTry) {
+    fn expr_try(&mut self, expr: &ExprTry, beginning_of_line: bool) {
         self.outer_attrs(&expr.attrs);
-        self.expr(&expr.expr);
+        self.expr_beginning_of_line(&expr.expr, beginning_of_line);
         self.word("?");
     }
 
-    fn subexpr_try(&mut self, expr: &ExprTry) {
-        self.subexpr(&expr.expr);
+    fn subexpr_try(&mut self, expr: &ExprTry, beginning_of_line: bool) {
+        self.subexpr(&expr.expr, beginning_of_line);
         self.word("?");
     }
 
@@ -787,9 +806,14 @@ impl Printer {
         match (iter.next(), iter.next()) {
             (
                 Some(
-                    expr @ (Expr::Closure(ExprClosure { attrs, .. })
+                    expr @ (Expr::Array(ExprArray { attrs, .. })
+                    | Expr::Async(ExprAsync { attrs, .. })
+                    | Expr::Block(ExprBlock { attrs, .. })
+                    | Expr::Closure(ExprClosure { attrs, .. })
                     | Expr::Struct(ExprStruct { attrs, .. })
-                    | Expr::Block(ExprBlock { attrs, .. })),
+                    | Expr::TryBlock(ExprTryBlock { attrs, .. })
+                    | Expr::Tuple(ExprTuple { attrs, .. })
+                    | Expr::Unsafe(ExprUnsafe { attrs, .. })),
                 ),
                 None,
             ) if !attr::has_outer(attrs) => {
@@ -816,7 +840,7 @@ impl Printer {
             self.inner_attrs(attrs);
             if let (Some(Stmt::Expr(expr)), None) = (block.stmts.get(0), block.stmts.get(1)) {
                 self.ibox(0);
-                self.expr(expr);
+                self.expr_beginning_of_line(expr, true);
                 self.end();
                 self.space();
             } else {
@@ -880,6 +904,13 @@ impl Printer {
             UnOp::Neg(_) => "-",
         });
     }
+
+    fn zerobreak_unless_short_ident(&mut self, beginning_of_line: bool, expr: &Expr) {
+        if beginning_of_line && is_short_ident(expr) {
+            return;
+        }
+        self.zerobreak();
+    }
 }
 
 pub fn requires_terminator(expr: &Expr) -> bool {
@@ -985,3 +1016,19 @@ fn needs_newline_if_wrap(expr: &Expr) -> bool {
         _ => false,
     }
 }
+
+fn is_short_ident(expr: &Expr) -> bool {
+    if let Expr::Path(expr) = expr {
+        if expr.attrs.is_empty()
+            && expr.qself.is_none()
+            && expr.path.leading_colon.is_none()
+            && expr.path.segments.len() == 1
+            && expr.path.segments[0].ident.to_string().len() as isize <= INDENT
+        {
+            if let PathArguments::None = expr.path.segments[0].arguments {
+                return true;
+            }
+        }
+    }
+    false
+}
